package net.udby.test.sdf;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Map;
import java.util.TimeZone;
import java.util.concurrent.ConcurrentHashMap;

public class DateFormatUtilClone {

	/** Map with SimpleDateFormat'ers */
	private static final Map<SdfCacheKey, SimpleDateFormat> fmtCache = new ConcurrentHashMap<SdfCacheKey, SimpleDateFormat>();

	private static synchronized DateFormat fmtCacheGet(SdfCacheKey key) {
		SimpleDateFormat sdf = fmtCache.get(key);
		if (sdf == null) {
			sdf = new SimpleDateFormat(key.formatString, key.locale);
			sdf.setTimeZone(key.timeZone);
			fmtCache.put(key, sdf);
		}
		return sdf;
	}

	private static DateFormat getCloned(SdfCacheKey key) {
		return (DateFormat) fmtCacheGet(key).clone();
	}

	private DateFormatUtilClone() {
	}

	public static String format(Date date, String formatString, Locale locale, TimeZone timeZone) {
		return getCloned(SdfCacheKey.c(formatString, locale, timeZone)).format(date);
	}

	public static String format(Date date, String formatString, Locale locale) {
		return format(date, formatString, locale, TimeZone.getDefault());
	}

	public static String format(Date date, String formatString) {
		return format(date, formatString, Locale.getDefault());
	}

	public static Date parse(String date, String formatString, Locale locale, TimeZone timeZone) throws ParseException {
		return getCloned(SdfCacheKey.c(formatString, locale, timeZone)).parse(date);
	}

	public static Date parse(String date, String formatString, Locale locale) throws ParseException {
		return parse(date, formatString, locale, TimeZone.getDefault());
	}

	public static Date parse(String date, String formatString) throws ParseException {
		return parse(date, formatString, Locale.getDefault());
	}
}
