package net.udby.test.sdf;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.udby.test.sdf.log.FirstFailure;
import net.udby.test.sdf.wrapper.DateFormatUtilClassSyncWrapper;
import net.udby.test.sdf.wrapper.DateFormatUtilCloneWrapper;
import net.udby.test.sdf.wrapper.DateFormatUtilCreateWrapper;
import net.udby.test.sdf.wrapper.DateFormatUtilThreadLocalWrapper;
import net.udby.test.sdf.wrapper.WrapperApi;
import net.udby.test.sdf.wrapper.stat.StatInfo;

public class TestDriver1 {

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		// warmup...
		System.out.println("warmup 1");
		runTest(DateFormatUtilClassSyncWrapper.class, 1, 10000l, 0);
		System.out.println("warmup 2");
		runTest(DateFormatUtilCreateWrapper.class, 1, 10000l, 0);
		System.out.println("warmup 3");
		runTest(DateFormatUtilCloneWrapper.class, 1, 10000l, 0);
		System.out.println("warmup 4");
		runTest(DateFormatUtilThreadLocalWrapper.class, 1, 10000l, 0);

		System.out.println("run");

		FirstFailure.clear();

		runTest(DateFormatUtilClassSyncWrapper.class, 4, 10000l, 0);

		FirstFailure.clear();

		runTest(DateFormatUtilCreateWrapper.class, 4, 10000l, 0);

		FirstFailure.clear();

		runTest(DateFormatUtilCloneWrapper.class, 4, 10000l, 0);

		FirstFailure.clear();

		runTest(DateFormatUtilThreadLocalWrapper.class, 4, 10000l, 0);

		System.out.println("done");
	}

	private static <T extends WrapperApi> Map<String, StatInfo> runTest(Class<T> clazz, int cnt, long runtime, long relaxtime)
			throws InterruptedException {
		final Object sync = new Object();
		List<TestRunner<T>> runners = new ArrayList<TestRunner<T>>(cnt);
		List<Thread> threads = new ArrayList<Thread>(cnt);
		for (int i = 0; i < cnt; i++) {
			TestRunner<T> runner = new TestRunner<T>(clazz, runtime, relaxtime, sync);
			runners.add(runner);
			Thread thread = new Thread(runner, "testing-" + clazz.getSimpleName() + "-" + (i + 1));
			threads.add(thread);
		}
		synchronized (sync) {
			for (Thread thread : threads) {
				thread.start();
			}
			sync.wait(100l);
			sync.notifyAll();
		}
		for (Thread thread : threads) {
			thread.join();
		}

		if (FirstFailure.failed()) {
			System.out.println(clazz.getSimpleName() + ": FAILED");
			System.out.println(FirstFailure.getInstance().toString());
			return null;
		}

		Map<String, StatInfo> methodStats = new HashMap<String, StatInfo>();
		for (TestRunner<T> runner : runners) {
			for (Map.Entry<String, StatInfo> entry : runner.getHandler().methodStats.entrySet()) {
				StatInfo si = methodStats.get(entry.getKey());
				if (si == null) {
					methodStats.put(entry.getKey(), entry.getValue());
				} else {
					methodStats.put(entry.getKey(), StatInfo.consolidate(entry.getValue(), si));
				}
			}
		}
		System.out.println(clazz.getSimpleName() + ": " + methodStats);
		return methodStats;
	}
}
