package net.udby.test.sdf;

import java.lang.reflect.Proxy;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

import net.udby.test.sdf.log.FirstFailure;
import net.udby.test.sdf.wrapper.WrapperApi;
import net.udby.test.sdf.wrapper.stat.StatWrapper;

public class TestRunner<T extends WrapperApi> implements Runnable {
	private static final String FORMAT_1 = "dd-MM-yyyy HH:mm:ss";

	private final T target;
	private final StatWrapper handler;
	private final WrapperApi proxy;
	private final long runTimeMs;
	private final Object sync;

	public TestRunner(Class<T> clazz, long runTimeMs, long relaxTimeMs, Object sync) {
		try {
			this.target = clazz.newInstance();
			this.handler = new StatWrapper(target, relaxTimeMs);
			this.proxy = (WrapperApi) Proxy.newProxyInstance(getClass().getClassLoader(), new Class<?>[] { WrapperApi.class }, handler);
			this.runTimeMs = runTimeMs;
			this.sync = sync;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	@Override
	public void run() {
		try {
			final Locale locale = Locale.getDefault();
			final TimeZone timeZone = TimeZone.getDefault();

			synchronized (sync) {
				sync.wait();
			}

			final SimpleDateFormat sdf = new SimpleDateFormat(FORMAT_1, locale);
			sdf.setTimeZone(timeZone);

			long tend = System.currentTimeMillis() + runTimeMs;
			while (System.currentTimeMillis() < tend && !FirstFailure.failed()) {
				final String s = "01-01-2001 23:58:01";
				Date dt = proxy.parse(s, FORMAT_1);
				String ss = proxy.format(dt, FORMAT_1);
				if (!ss.equals(s)) {
					throw new Exception("1.expected=" + s + ",was=" + ss);
				}

				final Date tst = new Date();
				ss = proxy.format(tst, FORMAT_1, locale);
				dt = proxy.parse(ss, FORMAT_1, locale);
				if (Math.abs(tst.getTime() - dt.getTime()) >= 1000l) {
					throw new Exception("2.expected=" + tst + ",was=" + dt);
				}
				final String xs = sdf.format(new Date());
				dt = proxy.parse(xs, FORMAT_1, locale, timeZone);
				ss = proxy.format(dt, FORMAT_1, locale, timeZone);
				if (!ss.equals(xs)) {
					throw new Exception("3.expected=" + xs + ",was=" + ss);
				}
			}
		} catch (Exception ignored) {
			FirstFailure.getInstance().failure(this, ignored);
		}
	}

	public T getTarget() {
		return target;
	}

	public StatWrapper getHandler() {
		return handler;
	}
}
