package net.udby.test.sdf.simple;

import java.lang.reflect.Proxy;
import java.util.Date;

import net.udby.test.sdf.log.FirstFailure;
import net.udby.test.sdf.wrapper.stat.StatWrapper;

public class TestRunner<T extends SimpleApi> implements Runnable {
	private final T target;
	private final StatWrapper handler;
	private final SimpleApi proxy;
	private final long runTimeMs;
	private final Object sync;

	public TestRunner(Class<T> clazz, long runTimeMs, long relaxTimeMs, Object sync) {
		try {
			this.target = clazz.newInstance();
			this.handler = new StatWrapper(target, relaxTimeMs);
			this.proxy = (SimpleApi) Proxy.newProxyInstance(getClass().getClassLoader(), new Class<?>[] { SimpleApi.class }, handler);
			this.runTimeMs = runTimeMs;
			this.sync = sync;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	@Override
	public void run() {
		try {
			synchronized (sync) {
				sync.wait();
			}

			long tend = System.currentTimeMillis() + runTimeMs;
			while (System.currentTimeMillis() < tend && !FirstFailure.failed()) {
				final String s = "01-01-2001 23:58:01";
				Date dt = proxy.parse(s);
				String ss = proxy.format(dt);
				if (!ss.equals(s)) {
					throw new Exception("1.expected=" + s + ",was=" + ss);
				}

				final Date tst = new Date();
				ss = proxy.format(tst);
				dt = proxy.parse(ss);
				if (Math.abs(tst.getTime() - dt.getTime()) >= 1000l) {
					throw new Exception("2.expected=" + tst + ",was=" + dt);
				}
			}
		} catch (Exception ignored) {
			FirstFailure.getInstance().failure(this, ignored);
		}
	}

	public T getTarget() {
		return target;
	}

	public StatWrapper getHandler() {
		return handler;
	}
}
