package net.udby.test.sdf.wrapper.stat;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import net.udby.test.sdf.log.FirstFailure;

public class StatWrapper implements InvocationHandler {
	private final Object wrapped;
	private final long relaxTimeMs;

	public final Map<String, StatInfo> methodStats = new HashMap<String, StatInfo>();

	public StatWrapper(Object wrapped, long relaxTimeMs) {
		this.wrapped = wrapped;
		this.relaxTimeMs = relaxTimeMs;
	}

	@Override
	public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
		try {
			long t0 = System.nanoTime();
			Object result = method.invoke(wrapped, args);
			long t1 = System.nanoTime();
			reportStats(method.getName(), t1 - t0);
			if (relaxTimeMs > 0) {
				// relax a bit...
				Thread.sleep(relaxTimeMs);
			}
			return result;
		} catch (Exception e) {
			FirstFailure.getInstance().failure(this, e);
			e.printStackTrace();
			throw e;
		}
	}

	private void reportStats(String methodSignature, long timeNs) {
		StatInfo si = methodStats.get(methodSignature);
		if (si == null) {
			si = new StatInfo(methodSignature, timeNs);
			methodStats.put(methodSignature, si);
		} else {
			si.add(timeNs);
		}
	}
}
